"""Constants for NZBGet."""
DOMAIN = "nzbget"

# Attributes
ATTR_SPEED = "speed"

# Data
DATA_COORDINATOR = "corrdinator"
DATA_UNDO_UPDATE_LISTENER = "undo_update_listener"

# Defaults
DEFAULT_NAME = "NZBGet"
DEFAULT_PORT = 6789
DEFAULT_SCAN_INTERVAL = 5  # time in seconds
DEFAULT_SPEED_LIMIT = 1000  # 1 Megabyte/Sec
DEFAULT_SSL = False
DEFAULT_VERIFY_SSL = False

# Services
SERVICE_PAUSE = "pause"
SERVICE_RESUME = "resume"
SERVICE_SET_SPEED = "set_speed"
