"""Define constants for the Ambient PWS component."""
import logging

DOMAIN = "ambient_station"
LOGGER = logging.getLogger(__package__)

ATTR_LAST_DATA = "last_data"

CONF_APP_KEY = "app_key"

TYPE_SOLARRADIATION = "solarradiation"
TYPE_SOLARRADIATION_LX = "solarradiation_lx"
