"""Constants for Ampio Air Quality platform."""
from datetime import timedelta
from typing import Final

ATTRIBUTION: Final = "Data provided by Ampio"
CONF_STATION_ID: Final = "station_id"
SCAN_INTERVAL: Final = timedelta(minutes=10)
