"""Constants for the Tesla Wall Connector integration."""

DOMAIN = "tesla_wall_connector"
DEFAULT_SCAN_INTERVAL = 30

WALLCONNECTOR_SERIAL_NUMBER = "serial_number"

WALLCONNECTOR_DATA_VITALS = "vitals"
WALLCONNECTOR_DATA_LIFETIME = "lifetime"

WALLCONNECTOR_DEVICE_NAME = "Tesla Wall Connector"
