"""Constants for the tolo integration."""

DOMAIN = "tolo"
DEFAULT_NAME = "TOLO Sauna"

DEFAULT_RETRY_TIMEOUT = 1
DEFAULT_RETRY_COUNT = 3

DEFAULT_MAX_TEMP = 60
DEFAULT_MIN_TEMP = 20

DEFAULT_MAX_HUMIDITY = 99
DEFAULT_MIN_HUMIDITY = 60

POWER_TIMER_MAX = 60
SALT_BATH_TIMER_MAX = 60
FAN_TIMER_MAX = 60
